import { Dispatch } from 'redux'

import session from '@agiliza/redux/session'
import { MainUseCase, RequestDriver, SecurityManager, UseCaseMessageType } from '@curio/client'
import { ConnectionError, DestinataryNotFoundError } from '@curio/client/errors'
import { ABORT } from '@curio/client/utils/constants'

const STORAGE_SESSION_KEY = '@app-exames-adm-session-token'

export interface ConfigService {
  url: string
  server: string
  system: number
  port: number
  module: number
  version: number
}

class Session extends MainUseCase {
  module: number | undefined
  version: number | undefined

  constructor(token, service, driver) {
    super(token, service, driver)
    this.module = service.module
    this.version = service.version
  }
}

export class AnonymousSession extends MainUseCase {}

export class SessionManager extends SecurityManager {
  public session: Session | undefined
  private _dispatch: Dispatch
  private _storage: Storage
  private _service: ConfigService
  constructor(storage: Storage, service: ConfigService, driver: RequestDriver, dispatch: Dispatch) {
    super(service, driver)
    this._dispatch = dispatch
    this._storage = storage
    this._service = service

    const token = this._storage.getItem(STORAGE_SESSION_KEY)

    const { actions } = session

    if (process.env.NODE_ENV === 'development') {
      dispatch(actions.setServiceInfo(service))
    }

    if (token) {
      dispatch(actions.authRequest())
      this.connectMainUseCase(new Session(token, this.service, driver))
      this.session!.timeoutCheck()
        .then(() => dispatch(actions.authSuccess()))
        .catch((err) => {})
    }
  }

  public anonymousConnection() {
    return super.openMainUseCase('Administrador', '', AnonymousSession)
  }

  public async openMainUseCase<U extends typeof MainUseCase>(username: string, password: string) {
    this.session = await super.openMainUseCase(username, password, Session)
    this.session.module = this._service.module
    this.session.version = this._service.version
    this._storage.setItem(STORAGE_SESSION_KEY, this.session.token.toString())

    this.session.addListener(ABORT, () => {
      this._unauthenticate(false)
    })
    this.session.addListener(UseCaseMessageType.RESPONSE, (message: any) => {
      if (message.error instanceof DestinataryNotFoundError || message.error instanceof ConnectionError) {
        this._unauthenticate(true)
      }
    })
    return this.session as InstanceType<U>
  }

  public connectMainUseCase(mainUseCase: Session) {
    this.session = mainUseCase
    if (!(mainUseCase instanceof AnonymousSession)) {
      this._storage.setItem(STORAGE_SESSION_KEY, this.session.token.toString())
    }

    this.session.addListener(ABORT, () => {
      this._unauthenticate(false)
    })
    this.session.addListener(UseCaseMessageType.RESPONSE, (message: any) => {
      if (message.error instanceof DestinataryNotFoundError || message.error instanceof ConnectionError) {
        this._unauthenticate(true)
      }
    })
    return super.connectMainUseCase(mainUseCase)
  }
  private _unauthenticate(expired: boolean) {
    this.session = undefined
    this._storage.removeItem(STORAGE_SESSION_KEY)
    if (expired) {
      this._dispatch(session.actions.logout())
    }
  }
}
