import { Dispatch } from 'redux'

import { store } from '@agiliza/redux'
import { FetchLink } from '@curio/client/links'
import { V3RequestParser } from '@curio/client/parsers'
import { createV3Validator } from '@curio/client/validators'

// import config from '../../config/config.json'
import { ConfigService, SessionManager } from './SessionManager'

interface Config {
  service: ConfigService
  accessToken: string
  resources: {
    get: string
    put: string
  }
  logs: boolean
  externalApiUrl: string
}

const createSessionManager = (config: Config, dispatch: Dispatch) => {
  const link = new FetchLink(config.service)
  const requestParser = new V3RequestParser(config.service)
  const sessionManager = new SessionManager(
    localStorage,
    config.service,
    (request) => {
      return Promise.resolve(request)
        .then(requestParser.parse)
        .then(JSON.stringify)
        .then(link.parse)
        .then(link.post)
        .then((response) => response.json())
        .then(createV3Validator(request).validate)
    },
    dispatch
  )

  interface Attributes {
    [index: string]: string
    _Id: string
    _Recipient: string
    _Sender: string
    _SerialNumber: string
    _SignalName: string
  }

  if (config.logs) {
    sessionManager[Symbol.observable]().subscribe({
      next: (msg) => {
        if (msg.signalName === '116' || msg.signalName === '120' || msg.signalName === '134' || !Number(msg.signalName)) {
          const signalName = msg.signalName.substr(msg.signalName.indexOf('_') + 1).replace(/_/g, ' ')
          const filteredBody = Object.keys(msg.body ? (msg.body as Attributes) : {}).reduce((acc, curr) => {
            if (curr.indexOf('_') < 0) return { ...acc, [curr]: (msg.body as Attributes)[curr] }
            else return acc
          }, {})

          const colorType = msg.type === 'response' ? 'green' : 'blue'
          console.log(
            `%c ${msg.type.toUpperCase()} %c ${signalName}`,
            `color: white; font-weight: bolder; background: ${colorType}`,
            'font-weight: bolder; font-style: italic;',
            Object.keys(filteredBody).length !== 0 ? filteredBody : ''
          )
        }
      },
      error: console.error,
      complete: () => console.log('complete'),
    })
  }

  return sessionManager
}

const CONFIG_PATH = './config.json'

let sm: SessionManager
let config: Config
async function getSessionManager() {
  if (sm) {
    return sm
  } else {
    try {
      const cnfg = await (await fetch(CONFIG_PATH)).json()
      config = cnfg
      sm = createSessionManager(config, store.dispatch)
      return sm
    } catch (e) {
      if (e instanceof Response) {
        //Not found, most likely
        throw process.env.NODE_ENV === 'production'
          ? 'Não foi encontrado o arquivo config.json na raiz da aplicação.'
          : 'Arquivo config.json faltando na pasta public.'
      }
      throw e
    }
  }
}

const wd: any = window
wd.__CURIO_GET_SESSION__ = () => getSessionManager().then((sm) => sm.session)

export { createSessionManager, getSessionManager, config }
