import deepmerge from 'deepmerge'

import { PayloadAction } from '@reduxjs/toolkit'

export const values = (obj: Record<string, unknown>) => {
  return Object.values(obj || {})
}

export const entries = (obj: Record<string, unknown>) => {
  return Object.entries(obj || {})
}
export const keys = (obj: Record<string, unknown>) => {
  return Object.keys(obj || {})
}

export const extractNumbers = (str?: string): string => (str ? str.replace(/\D/g, '') : '')
export const extractLetters = (str?: string): string =>
  str
    ? str
        .replace(/[^A-Za-z ]/g, '')
        .toUpperCase()
        .trim()
    : ''

export const getTypesActions = <Actions>(actions: Actions) => {
  type Types = {
    [key in keyof Actions]: string
  }
  return Object.keys(actions).reduce((acc, curr) => {
    return { ...acc, [curr]: `${actions[curr as keyof Actions]}` }
  }, {} as Types)
}

export const getError = (error: Error | { error: string } | string) => {
  if (error instanceof Error) {
    if (process.env.NODE_ENV === 'development') return `${error.message} - ${error.stack}`
    return error.message || 'Ocorreu um erro na requisição ao servidor.'
  } else if (typeof error === 'object') return error.error
  else return error
}

export const getTablePaperHeight = (heightReso: number) => {
  if (650 <= heightReso && heightReso <= 800) return heightReso / 1.65
  if (980 <= heightReso && heightReso <= 1100) return heightReso / 1.35
  return heightReso / 1.2
  // switch (heightReso) {
  //   // case 720:
  //   //   return 445
  //   // case 1080:
  //   //   return 800
  //   default:
  //     return heightReso / 1.65
  // }
}

export const mergeState = <State extends { byId: Record<string, DataType>; allIds: string[] }, DataType extends { id: string }>(
  state: State,
  payload: DataType[]
) => {
  const { allIds, byId } = state
  payload.forEach((p) => {
    if (!byId[p.id]) allIds.push(p.id)
    byId[p.id] = p
  })
}
type ActionType<Type, Payload> = {
  type: Type
  payload: Payload
}

export const reducerActions = <StateType extends { fetching: { [x: string]: any } | boolean }, Action extends { type: string; payload: any }>(
  name: string
): {
  [k in Action['type']]: (state: StateType, action: PayloadAction<Action['payload'] | undefined>) => void
} => {
  const request = `${name}Request`
  const success = `${name}Success`
  const error = `${name}Error`

  return {
    [request]: (state: StateType, action: PayloadAction<Action['payload']>) => {
      // eslint-disable-line
      if (typeof state.fetching === 'object') state.fetching[name] = true
      else state.fetching = true
    },
    [success]: (state: StateType) => {
      if (typeof state.fetching === 'object') state.fetching[name] = false
      else state.fetching = false
    },
    [error]: (state: StateType) => {
      if (typeof state.fetching === 'object') state.fetching[name] = false
      else state.fetching = false
    },
  } as {
    [k in Action['type']]: (state: StateType, action: PayloadAction<Action['payload'] | undefined>) => void
  }
}

export interface Entity {
  id: string
}

export interface NormalizedEntity<T> {
  [id: string]: T
}

export interface EntityStore<T> {
  byId: NormalizedEntity<T>
  allIds: string[]
}

export function syncById<T extends Entity, State extends EntityStore<T>>(state: State, entities: NormalizedEntity<T> = {}): NormalizedEntity<T> {
  return deepmerge<NormalizedEntity<T>>(state.byId, entities, {
    arrayMerge: <Type>(dest: Type[], src: Type[]) => src,
  })
}

export function deepmergeArray<T = any>(array1: T[], array2: T[]) {
  return array2.reduce<T[]>((prev, curr) => (prev.includes(curr) ? prev : [...prev, curr]), array1)
}

export function syncAllIds<T extends Entity, State extends EntityStore<T>>(state: State, entities: NormalizedEntity<T> = {}): string[] {
  return deepmergeArray(state.allIds, Object.keys(entities))
}

type FirstParameter<T extends (...args: any) => any> = Parameters<T>[0]
type RequiredBy<T, K extends keyof T> = Pick<T, K> & Partial<Omit<T, K>>

export type { ActionType, FirstParameter, RequiredBy }
