import React, { useState } from 'react'
import { RouteComponentProps } from 'react-router'

import { extractCurrency, extractNumbers } from '@agiliza/utils/extractors'
import { formatCurrency } from '@agiliza/utils/formatters'
import {
  Button,
  CircularProgress,
  InputAdornment,
  TextareaAutosize,
  TextField,
  Typography
} from '@material-ui/core'
import { withStyles, WithStyles } from '@material-ui/core/styles'
import { Autocomplete } from '@material-ui/lab'

import { creditLines } from '../../../../__mocks__/creditLines'
import { CREDIT_LINES_PATHS } from '../router'
import SliderField from './SliderField'
import styles from './styles'

type CreditLine = typeof creditLines[0]

type ExtendedProps = WithStyles<typeof styles> & RouteComponentProps

interface Props extends ExtendedProps {
  fetching: boolean
}

const CreditLinesList: React.FC<Props> = (props) => {
  const { classes, fetching, history, match } = props

  const [selectedCreditLine, setSelectedCreditLine] = useState<CreditLine | null>(null)
  const [amount, setAmount] = useState('0')
  const [paymentMonths, setPaymentMonths] = useState(0)
  const [graceMonths, setGraceMonths] = useState(0)

  const parentPath = match.path.substring(0, match.path.lastIndexOf('/'))
  const handleSimulate = () => {
    if (selectedCreditLine) history.push({ pathname: parentPath + CREDIT_LINES_PATHS.simulation, state: { id: selectedCreditLine.id } })
  }

  return (
    <div className={classes.pageContent}>
      <div className={classes.contentContainer}>
        <div className={classes.content}>
          <Typography variant="h5" className={classes.title}>
            Escolha a melhor linha de crédito para você e clique em simular!
          </Typography>
          <Autocomplete<CreditLine>
            id="credit-lines"
            className={classes.creditLinesAutocomplete}
            options={creditLines}
            getOptionLabel={(option) => option.name}
            getOptionSelected={(opt, val) => opt.id === selectedCreditLine?.id}
            noOptionsText="Nenhuma linha encontrado"
            value={selectedCreditLine}
            onChange={(_, opt) => {
              setSelectedCreditLine(opt || null)

              setAmount(opt?.amount.min.toString() || '0')
              setPaymentMonths(opt?.paymentMonths.min || 0)
              setGraceMonths(opt?.graceMonths.min || 0)
            }}
            renderInput={(params) => {
              const isValid = React.isValidElement(params.InputProps.endAdornment)
              return (
                <TextField
                  {...params}
                  id="select-patient-id"
                  label="Selecione ou Digite Id Paciente"
                  variant="outlined"
                  InputProps={{
                    ref: params.InputProps.ref,
                    endAdornment: fetching ? (
                      <InputAdornment position="end" style={{ flex: '1', display: 'flex', justifyContent: 'flex-end' }}>
                        <CircularProgress size={20} />
                      </InputAdornment>
                    ) : isValid ? (
                      React.cloneElement(params.InputProps.endAdornment as React.ReactElement, {
                        style: { marginRight: '1rem' },
                      })
                    ) : (
                      params.InputProps.endAdornment
                    ),
                  }}
                />
              )
            }}
          />
          <TextField
            className={classes.description}
            label="Descrição da linha de crédito"
            value={selectedCreditLine?.informations || ''}
            rows={5}
            multiline
            contentEditable={false}
            InputLabelProps={{
              shrink: true,
            }}
            InputProps={{ readOnly: true, inputProps: { component: TextareaAutosize } }}
            variant="outlined"
            unselectable="on"
          />
          <SliderField
            className={classes.sliderField}
            title="De quanto você precisa?"
            min={selectedCreditLine?.amount.min || 0}
            max={selectedCreditLine?.amount.max || 5000}
            marks={
              selectedCreditLine
                ? [
                    { value: selectedCreditLine.amount.min, label: formatCurrency(selectedCreditLine.amount.min.toString()) },
                    { value: selectedCreditLine.amount.max, label: formatCurrency(selectedCreditLine.amount.max.toString()) },
                  ]
                : [
                    { value: 0, label: formatCurrency('0') },
                    { value: 5000, label: '' },
                  ]
            }
            valueSlider={Number(amount)}
            valueField={formatCurrency(amount.toString(), true)}
            step={0.01}
            disabled={!selectedCreditLine}
            onChangeSlider={(_, value) => setAmount((value as number).toString())}
            onChangeField={(evt) => setAmount(extractCurrency(evt.target.value))}
            onBlurField={(evt) => {
              const amountValue = extractCurrency(evt.target.value)
              if (selectedCreditLine)
                setAmount(Number(amountValue) <= selectedCreditLine.amount.max ? amountValue : selectedCreditLine.amount.max.toString())
            }}
          />
          <SliderField
            className={classes.sliderField}
            title="Quantos meses para pagar?"
            min={selectedCreditLine?.paymentMonths.min || 0}
            max={selectedCreditLine?.paymentMonths.max || 12}
            marks={
              selectedCreditLine
                ? [
                    { value: selectedCreditLine.paymentMonths.min, label: `${selectedCreditLine.paymentMonths.min} meses` },
                    { value: selectedCreditLine.paymentMonths.max, label: `${selectedCreditLine.paymentMonths.max} meses` },
                  ]
                : [
                    { value: 0, label: '0 meses' },
                    { value: 12, label: '' },
                  ]
            }
            valueSlider={Number(paymentMonths)}
            valueField={paymentMonths.toString() + ' meses'}
            disabled={!selectedCreditLine}
            onChangeSlider={(_, value) => setPaymentMonths(value as number)}
            onChangeField={(evt) => setPaymentMonths(Number(evt.target.value))}
            onBlurField={(evt) => {
              const paymentMonthsValue = Number(extractNumbers(evt.target.value))
              if (selectedCreditLine)
                setPaymentMonths(
                  paymentMonthsValue <= selectedCreditLine.paymentMonths.max ? paymentMonthsValue : selectedCreditLine.paymentMonths.max
                )
            }}
          />
          <SliderField
            className={classes.sliderField}
            title="Quantos meses de carência?"
            min={selectedCreditLine?.graceMonths.min || 0}
            max={selectedCreditLine?.graceMonths.max || 12}
            marks={
              selectedCreditLine
                ? [
                    { value: selectedCreditLine.graceMonths.min, label: `${selectedCreditLine.graceMonths.min} meses` },
                    { value: selectedCreditLine.graceMonths.max, label: `${selectedCreditLine.graceMonths.max} meses` },
                  ]
                : [
                    { value: 0, label: '0 meses' },
                    { value: 12, label: '' },
                  ]
            }
            valueSlider={Number(graceMonths)}
            valueField={graceMonths.toString() + ' meses'}
            disabled={!selectedCreditLine}
            onChangeSlider={(_, value) => setGraceMonths(value as number)}
            onChangeField={(evt) => setGraceMonths(Number(evt.target.value))}
            onBlurField={(evt) => {
              const graceMonthsValue = Number(extractNumbers(evt.target.value))
              if (selectedCreditLine)
                setGraceMonths(graceMonthsValue <= selectedCreditLine.graceMonths.max ? graceMonthsValue : selectedCreditLine.graceMonths.max)
            }}
          />
          <div className={classes.btnContainer}>
            <Button disabled={!selectedCreditLine} variant="contained" color="secondary" onClick={handleSimulate}>
              Simular
            </Button>
          </div>
        </div>
      </div>
    </div>
  )
}

export default withStyles(styles)(CreditLinesList)
