import React, { useEffect, useState } from 'react'

import { formatPhone } from '@agiliza/utils/formatters'
import {
  getErrorProps,
  invalidState,
  useErrorValidator,
  Validators,
  validState
} from '@agiliza/utils/hooks/errorValidation'
import { isValidEmail, isValidPhone } from '@agiliza/utils/validators'
import { TextField, TextFieldProps } from '@material-ui/core'
import { withStyles, WithStyles } from '@material-ui/core/styles'

import styles from './styles'

interface UserFormFieldProps extends Pick<TextFieldProps, 'label' | 'value' | 'onChange' | 'error' | 'helperText' | 'inputProps'> {}

const UserFormField = ({ helperText, ...props }: UserFormFieldProps) => <TextField fullWidth margin="normal" variant="outlined" {...props} />

interface State {
  name: string
  phoneNumber: string
  email: string
}

const validators: Validators<Partial<State>> = {
  phoneNumber: (vl) => (isValidPhone(vl) ? validState(vl) : invalidState(vl, '')),
  email: (vl) => (isValidEmail(vl) ? validState(vl) : invalidState(vl, '')),
}

type ExtendedProps = WithStyles<typeof styles>

interface Props extends ExtendedProps {
  onChangeCanGoForward: (vl: boolean) => void
}

const UserForm = (props: Props) => {
  const { classes, onChangeCanGoForward } = props
  const [state, setState] = useState<State>({ email: '', phoneNumber: '', name: '' })
  const { name, phoneNumber, email } = state

  const { errorState, validState: isValidState, actions } = useErrorValidator({ email: '', phoneNumber: '', name: '' }, validators)

  useEffect(() => {
    onChangeCanGoForward(isValidState)
  }, [])

  useEffect(() => {
    onChangeCanGoForward(isValidState)
  }, [isValidState])

  const handleChange =
    (key: keyof State): TextFieldProps['onChange'] =>
    (evt) => {
      setState({ ...state, [key]: evt.target.value })
      actions.validate({ [key]: evt.target.value })
    }

  return (
    <div className={classes.contentContainer}>
      <div className={classes.content}>
        <UserFormField label="Nome" value={name} onChange={handleChange('name')} {...getErrorProps(errorState.name)} />
        <UserFormField
          label="Número de celular"
          value={formatPhone(phoneNumber)}
          onChange={handleChange('phoneNumber')}
          inputProps={{ maxLength: 15 }}
          {...getErrorProps(errorState.phoneNumber)}
        />
        <UserFormField label="Email" value={email} onChange={handleChange('email')} {...getErrorProps(errorState.email)} />
      </div>
    </div>
  )
}

export default withStyles(styles)(UserForm)
